package Constants;
use strict;
use warnings;
use Data::Dumper;

use constant SCALAR => 'SCALAR';

our %DEFAULTS = 
(
    factory     =>  {
                        GENERAL_logfile         =>  "var/log/factory.log",
                        GENERAL_modpath         =>  "Modules",
                     },

    receiver    =>  {
                        GENERAL_method          =>  ( "stdout" ),
                        GENERAL_logfile         =>  "var/log/receiver.log",
                        GENERAL_wardenpath      =>  "/opt/warden/client",
                        GENERAL_requested_type  =>  "_all_",
                        
                        FILE_directory          =>  "var/fileout/",
                        FILE_method             =>  "append",
                        FILE_appendfilename     =>  "received",
                        FILE_extension          =>  "csv",
                        
                        DB_dbengine               =>  "sqlite",
                    },


    db          =>  {
                        SQLITE_db   =>  "var/db.sqlite",
                        SQLITE_user =>  "",
                        SQLITE_pass =>  "",
                        
                        MYSQL_db    =>  "warden",
                        MYSQL_user  =>  "root",
                        MYSQL_pass  =>  "",
                        MYSQL_host  =>  "localhost",
                        MYSQL_port  =>  "3306",
                    },

    cleaner     =>  {
                        GENERAL_method  =>  ( "db" ),
                        GENERAL_maxage  =>  "11D",
                    },
);


sub getDefaultValue {
    my ($valuename, $section) = @_;
    
    $valuename =~ s/\./_/g;
    my @value = $DEFAULTS{$section}{$valuename};
    die "Value '$valuename' is not defined" if not @value;
    
    return (wantarray ? @value : $value[0]);
}

sub assignValue {
    my ($valuename, $cfg, $section, $nocheck) = @_;
    
    my @configvalue = $cfg->param($valuename);
    $valuename =~ s/\./_/g;

    my @value;

    if(!defined $nocheck) { 
        my @defaultvalue = getDefaultValue($valuename, $section);
        @value = (@configvalue ? @configvalue : @defaultvalue);
    } 
    else {
        @value = @configvalue;     
    }
    die "Value '$valuename' is not defined" if not defined $value[0];

    if(wantarray and $value[0] eq "") {
        return ();
    }
    else {
        return (wantarray ? @value : $value[0]);
    }
}


sub mergeConfigs {
    my ($config, $section, $constants, $format) = @_;
    my %ret;
 
    my $conf_hash = $config->get_block($section);

    foreach my $const_key ( keys %$constants )
    {
        if( exists $conf_hash->{$const_key} ) {
            if(ref($constants->{$const_key}) eq ref($conf_hash->{$const_key})) {
                $ret{$const_key} = $conf_hash->{$const_key};
            }
            elsif (ref($constants->{$const_key}) eq 'ARRAY') {
                $ret{$const_key} = (defined $conf_hash->{$const_key} ? [$conf_hash->{$const_key}] : []);
            }
            elsif (ref(\$constants->{$const_key}) eq 'SCALAR') {
                $ret{$const_key} = (defined $conf_hash->{$const_key} ? $conf_hash->{$const_key}->[0] : "");
            } 

            if(exists $format->{$const_key}) {
                if($ret{$const_key} !~ $format->{$const_key}) {
                    $ret{$const_key} = $constants->{$const_key};
                }
            }
        }
        else {
            $ret{$const_key} = $constants->{$const_key};
        }
    }

    return \%ret;
}
