#!/usr/bin/perl
use strict;
use warnings;

use FindBin qw($Bin);
use lib "$Bin/../";

use WardenApp::DB;
use WardenApp::Factory;
use Config::Simple;

use WardenApp::Constants;

use constant RECEIVER_SECTION  => 'receiver';
use constant DB_CONFIG_FILE => "$Bin/../etc/db.conf";
use constant RECV_CONFIG_FILE => "$Bin/../etc/receiver.conf";
use constant GENERAL_CONFIG_FILE => "$Bin/../etc/factory.conf";

my $cfg = new Config::Simple;
$cfg->read(GENERAL_CONFIG_FILE) or die $cfg->error();

my $cfg_rcv = new Config::Simple;
$cfg_rcv->read(RECV_CONFIG_FILE) or die $cfg_rcv->error();

my $cfg_db = new Config::Simple;
$cfg_db->read(DB_CONFIG_FILE) or die $cfg_db->error();

my $db_engine = Constants::assignValue("DB.dbengine",  $cfg_rcv, RECEIVER_SECTION);
my $dbh = DB::connectDB($cfg_db, $db_engine);
if(not defined $dbh) {
    exit 0;
}

if(defined $ARGV[0] and $ARGV[0] ne "") {
    if(Factory::runModule($ARGV[0], $cfg, $dbh, $db_engine)) {
        exit 1;
    }
    else {
        exit 0;
    }
}
else {
    print "Use module's name as parameter", "\n";
    exit 0;
}

DB::closeDB($dbh);

