#!/usr/bin/perl -w
#
# unregisterClient.pl
#
# Copyright (C) 2011-2012 Cesnet z.s.p.o
# Author(s): 	Tomas PLESNIK	<plesnik@ics.muni.cz>
#		Jan SOUKAL 	<soukal@ics.muni.cz>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Cesnet z.s.p.o nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# This software is provided ``as is'', and any express or implied
# warranties, including, but not limited to, the implied warranties of
# merchantability and fitness for a particular purpose are disclaimed.
# In no event shall the Cesnet z.s.p.o or contributors be liable for
# any direct, indirect, incidental, special, exemplary, or consequential
# damages (including, but not limited to, procurement of substitute
# goods or services; loss of use, data, or profits; or business
# interruption) however caused and on any theory of liability, whether
# in contract, strict liability, or tort (including negligence or
# otherwise) arising in any way out of the use of this software, even
# if advised of the possibility of such damage.

use strict;
use Getopt::Std;
use Switch;
use File::Basename;

our $VERSION = "0.1";

my $warden_path = '/opt/warden-server';
require $warden_path . '/lib/WardenReg.pm';
my $filename = basename($0);

#-------------------------------------------------------------------------------
#                               Functions
#-------------------------------------------------------------------------------
sub usage {
  print "Usage: $filename [-h -i <client_id>]\n";
  exit 1;
}

sub help {
  print "$filename [-h -i <client_id>]\n";
  print "-h     print this text and exit\n";
  print "-i     client_id for unregistration\n";
  exit 0;
}


#-------------------------------------------------------------------------------
# errMsg - print error message and die
#-------------------------------------------------------------------------------
sub errMsg
{
  my $msg = shift;
  $msg = trim($msg);
  print $msg . "\n";
  exit 1;
} # End of errMsg


#-------------------------------------------------------------------------------
# trim - remove whitespace from the start and end of the string
#-------------------------------------------------------------------------------
sub trim
{
  my $string = shift;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;
  return $string;
} # End of trim


#-------------------------------------------------------------------------------
#                               MAIN
#-------------------------------------------------------------------------------
our ($opt_h, $opt_i);

if ($#ARGV == -1) {usage}

die usage unless getopts("i:h");
my $client_id	= $opt_i;
my $help        = $opt_h;

# catch help param
if ($help) {
	  help;
}

# superuser controle
my $UID = $<;
if ($UID != 0) {die errMsg("You must be root for running this script!")}

# check parameters definition
if (!defined $client_id) {
  print "ERROR: Parameter 'client_id' is not defined!\n";
  exit 1;
}

my $return = WardenReg::unregisterClient($warden_path, $client_id);
$return ? print "Unregistration of client (#$client_id) was SUCCESSFUL...\n" : print "Unregistration of client (# $client_id) FAILED!\n";

exit 0;
