#!/usr/bin/perl -w
#
# getStatus.pl
#
# Copyright (C) 2011-2012 Cesnet z.s.p.o
#
# Use of this source is governed by a BSD-style license, see LICENSE file.

use strict;
use Getopt::Std;
use File::Basename;

our $VERSION = "2.1";

my $warden_path = '/opt/warden-server';
require $warden_path . '/lib/WardenStatus.pm';
my $filename = basename($0);

#-------------------------------------------------------------------------------
#                               Functions
#-------------------------------------------------------------------------------
sub usage {
  print "Usage: $filename [without parameters]\n";
  exit 1;
}


#-------------------------------------------------------------------------------
# errMsg - print error message and die
#-------------------------------------------------------------------------------
sub errMsg
{
  my $msg = shift;
  $msg = trim($msg);
  print $msg . "\n";
  exit 1;
} # End of errMsg


#-------------------------------------------------------------------------------
# trim - remove whitespace from the start and end of the string
#-------------------------------------------------------------------------------
sub trim
{
  my $string = shift;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;
  return $string;
} # End of trim


#-------------------------------------------------------------------------------
#                               MAIN
#-------------------------------------------------------------------------------
our ($opt_h);

die usage unless getopts("h");
my $help = $opt_h;

# catch help param
if ($help) {
  usage;
}

# superuser controle
my $UID = $<;
if ($UID != 0) {
  die errMsg("You must be root for running this script!")
}

my @status = WardenStatus::getStatus($warden_path);

# take and remove first element of array @status and save it into $server_status_ref
my $server_status_ref = shift(@status);
my @server_status = @$server_status_ref;

my $syslog;
my $syslog_verbose;

if ($server_status[7] == 1) {
  $syslog = "enabled";
} else {
  $syslog = "disabled";
}

if ($server_status[8] == 1) {
  $syslog_verbose = "enabled";
} else {
  $syslog_verbose = "disabled";
}

print "Warden server variables:\n";
print "========================\n";
print "SERVER_VERSION:\t\t$server_status[0]\n";
print "HOSTNAME:\t\t$server_status[1]\n";
print "IP_ADDRESS:\t\t$server_status[2]\n";
print "PORT:\t\t\t$server_status[3]\n";
print "DB_NAME:\t\t$server_status[4]\n";
print "DB_USER:\t\t$server_status[5]\n";
print "DB_HOST:\t\t$server_status[6]\n";
print "SYSLOG:\t\t\t$syslog\n";
print "SYSLOG_VERBOSE:\t\t$syslog_verbose\n";
print "SYSLOG_FACILITY:\t$server_status[9]\n";
print "\n";

print "Warden server status:\n";
print "=====================\n";
print "Database size:\t\t\t$server_status[10]\n";
print "Count of saved events:\t\t$server_status[11]\n";
print "Last ID in events table:\t$server_status[12]\n";
print "Time of first inserted event:\t$server_status[13] (UTC)\n";
print "Time of latest inserted event:\t$server_status[14] (UTC)\n";
print "Count of registered clients:\t$server_status[15]\n";
print "\n";

# check if sum of registered client isn't 0
if ($server_status[15] != 0) {
  print "Statistics of active registered senders:\n";
  print "========================================\n";
  print "+----------------------------------------------------------------------------------------------------------------+\n";
  print "| Client ID  | Hostname                       | Service                   | Stored events | Last insertion (UTC) |\n";
  print "+----------------------------------------------------------------------------------------------------------------+\n";
  foreach my $client_status_ref (@status){
    my @client_status = @$client_status_ref;
    printf("| %-10s ", $client_status[0] || "unknown");
    printf("| %-30s ", $client_status[1] || "unknown");
    printf("| %-25s ", $client_status[2] || "unknown");
    printf("| %-13s ", $client_status[3] || "unknown");
    printf("| %-20s |\n", $client_status[4] || "unknown");
  }
  print "+----------------------------------------------------------------------------------------------------------------+\n";
  print "\n";
}

print "Current server status in:\t" . scalar localtime(time) . "\n";

exit 0;
