#!/usr/bin/perl -w
#
# registerSender.pl
#
# Copyright (C) 2011-2012 Cesnet z.s.p.o
#
# Use of this source is governed by a BSD-style license, see LICENSE file.

use strict;
use Getopt::Std;
use Switch;
use File::Basename;

our $VERSION = "2.0";

my $warden_path = '/opt/warden-server';
require $warden_path . '/lib/WardenReg.pm';
my $filename = basename($0);

#-------------------------------------------------------------------------------
#				Functions
#-------------------------------------------------------------------------------
sub usage {
  print "Usage: $filename [-h -n <hostname> -r <requestor> -s <service> -d <description_tags> -i <ip_net_client>]\n";
  exit 1;
}

sub help {
  print "$filename [-h -n <hostname> -r <requestor> -s <service> -d <description_tags> -i <ip_net_client>]\n";
  print "-h 	print this text and exit\n";
  print "-n 	hostname of sender\n";
  print "-r 	client registration requestor\n";
  print "-s 	name of service which sent events\n";
  print "-d 	description tags of send events\n";
  print "-i 	CIDR of sender\n";
  exit 0;
}


#-------------------------------------------------------------------------------
# errMsg - print error message and die
#-------------------------------------------------------------------------------
sub errMsg
{
  my $msg = shift;
  $msg = trim($msg);
  print $msg . "\n";
  exit 1;
} # End of errMsg


#-------------------------------------------------------------------------------
# trim - remove whitespace from the start and end of the string
#-------------------------------------------------------------------------------
sub trim
{
  my $string = shift;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;
  return $string;
} # End of trim


#-------------------------------------------------------------------------------
#				MAIN
#-------------------------------------------------------------------------------
our ($opt_n, $opt_r, $opt_s, $opt_d, $opt_i, $opt_h);

if ($#ARGV == -1) {usage}

die usage unless getopts("n:r:s:d:i:h");
my $hostname		= $opt_n;
my $requestor		= $opt_r;
my $service		= $opt_s;
my $description_tags 	= $opt_d;
my $ip_net_client	= $opt_i;
my $help 		= $opt_h;

# catch help param
if ($help) {
  help;
}

if ($ip_net_client !~ /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/(\d|[1-2]\d|3[0-2]))$/) {
    die errMsg("Enter correct IP in CIDR format!");
}

# superuser controle
my $UID = $<;
if ($UID != 0) {die errMsg("You must be root for running this script!")}

# check parameters definition
switch () {
  case {!defined $hostname} { print "ERROR: Parameter 'hostname' is not defined!\n"; exit 1; }
  case {!defined $requestor} { print "ERROR: Parameter 'requestor' is not defined!\n"; exit 1; }
  case {!defined $service} { print "ERROR: Parameter 'service' is not defined!\n"; exit 1; }
  case {!defined $description_tags} { print "ERROR: Parameter 'description_tags' is not defined!\n"; exit 1; }
  case {!defined $ip_net_client} { print "ERROR: Parameter 'ip_net_client' is not defined!\n"; exit 1; }
}

# register sender at warden server
my $return = WardenReg::registerSender($warden_path, $hostname, $requestor, $service, $description_tags, $ip_net_client);
$return ? print "Registration of $hostname was SUCCESSFUL...\n" : print "Registration of $hostname FAILED!\n";

exit 0;
